#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

import nucleotide.translator
import nucleotide.config
import nucleotide.klass

def _boost_blank_CPPPATH( P_list ):
    return [ os.getenv('BOOST_INCLUDE') ]

def _boost_blank_LIBPATH( P_list ):
    return [ os.getenv('BOOST_LIBPATH') ]

atom_boost_blank = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor' : 'X',
        'name'   : 'X',
        'version': 'X'
    },
    'config' : {
            'CPPPATH' : _boost_blank_CPPPATH,
            'LIBPATH' : _boost_blank_LIBPATH
    },
    'name' : 'boost:blank',
    'class':  [ 'boost', 'boost:blank' ]
}

class Boost:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'boost:blank',           atom_boost_blank            )

    @staticmethod
    def check():

        function.check__env( 'BOOST_INCLUDE' )
        function.check__env( 'BOOST_LIBPATH' )
        function.check__env( 'BOOST_VERSION' )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform
import traceback

import nucleotide
import nucleotide.component
import nucleotide.component.function
import nucleotide.component.windows.msvc.atom.module.generator


def _windows_msvc_atom_module_tbb_CPPDEFINES( P_list ):
    #print( "_windows_msvc_atom_module_tbb_CPPDEFINES" )
    Ir_result = [];
    return Ir_result

def _windows_msvc_atom_module_tbb_CPPPATH( P_list ):
    #print( "_windows_msvc_atom_module_tbb_CPPPATH" )

    prefix  = { "TBBROOT", "TBB_ROOT", "TBB" }
    Ir_result = nucleotide.component.windows.msvc.atom.module.generator.find_ENVIRONMENT_CPPPATH( prefix, P_list )

    return Ir_result

def _windows_msvc_atom_module_tbb_LINKFLAGS( P_list ):
    Ir_result = [];
    #print( '_windows_msvc_atom_module_tbb_LINKFLAGS' )
    return Ir_result

def _windows_msvc_atom_module_tbb_LIBPATH( P_list ):
    #print( "_windows_msvc_atom_module_tbb_LIBPATH" )

    prefix  = { "TBBROOT", "TBB_ROOT", "TBB" }
    Ir_result = nucleotide.component.windows.msvc.atom.module.generator.find_ENVIRONMENT_LIBPATH( prefix, P_list )

    return Ir_result

def _windows_msvc_atom_module_tbb_LIBS( P_list ):
    #print( "_windows_msvc_atom_module_tbb_LIBS" )
    Ir_result = [];
    return Ir_result


atom__common_package = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor' : 'Microsoft',
        'name'   : 'msvc',
        'version': 'X'
    },
    'config' : {
        'CPPDEFINES' : _windows_msvc_atom_module_tbb_CPPDEFINES,
        'CPPPATH'    : _windows_msvc_atom_module_tbb_CPPPATH,
        'LINKFLAGS'  : _windows_msvc_atom_module_tbb_LINKFLAGS,
        'LIBPATH'    : _windows_msvc_atom_module_tbb_LIBPATH,
        'LIBS'       : _windows_msvc_atom_module_tbb_LIBS,
    },

    'name' :'package',
    'class':  [ 'package::tbb', 'windows:package:tbb' ]
}





#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import copy

## Miscelenious custom data, organised in simple ( key, data ) pairs
class Custom:

    ## Don not use this member directly
    m_dictinary= {}

    def __init__( self, P_dictinary = {} ):
        self.m_dictinary = copy.deepcopy( P_dictinary )

    def get( self, P_name ):
        if( False == ( P_name in self.m_dictinary ) ):
            return None
        return self.m_dictinary[ P_name  ]

    def set( self, P_name, P_value ):
        self.m_dictinary[ P_name ] = P_value

    def exists( self, P_name ):
        if( False == ( P_name in self.m_dictinary ) ):
            return False
        return True

    def get_dictinary( self ):
        return self.m_dictinary



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 



# Scons Native thing only. Stored in (key,data) format
class Config:
    m_native = None

    def __init__( self, P_data = {} ):
        self.m_native = {
            #'TARGET_ARCH' : 'x86', #<purposely omit this key
            #'MSVC_VERSION': '12',  #<purposely omit this key
            #'CC'          : '',    #<purposely omit this key
            #'CXX'         : '',    #<purposely omit this key
            #'LINK'        : '',    #<purposely omit this key
            'CPPFLAGS'     : [],
            'CPPPATH'      : [],
            'CPPDEFINES'   : [],
            'LIBPATH'      : [],
            'LIBS'         : [],
            'LINKFLAGS'    : [],
        }

        #print( 'Config::__init__( P_data = ' + str( P_data ) + ' )' )

        if( True == ( 'CPPFLAGS'     in P_data ) ): self.m_native[ 'CPPFLAGS'   ] = P_data[ 'CPPFLAGS'  ]
        if( True == ( 'CPPPATH'      in P_data ) ): self.m_native[ 'CPPPATH'    ] = P_data[ 'CPPPATH'   ]
        if( True == ( 'CPPDEFINES'   in P_data ) ): self.m_native[ 'CPPDEFINES' ] = P_data[ 'CPPDEFINES']
        if( True == ( 'LIBPATH'      in P_data ) ): self.m_native[ 'LIBPATH'    ] = P_data[ 'LIBPATH'   ]
        if( True == ( 'LIBS'         in P_data ) ): self.m_native[ 'LIBS'       ] = P_data[ 'LIBS'      ]
        if( True == ( 'LINKFLAGS'    in P_data ) ): self.m_native[ 'LINKFLAGS'  ] = P_data[ 'LINKFLAGS' ]
        if( True == ( 'TARGET_ARCH'  in P_data ) ): self.m_native[ 'TARGET_ARCH'  ] = P_data[ 'TARGET_ARCH'  ]
        if( True == ( 'MSVC_VERSION' in P_data ) ): self.m_native[ 'MSVC_VERSION' ] = P_data[ 'MSVC_VERSION' ]
        if( True == ( 'CC'           in P_data ) ): self.m_native[ 'CC'   ] = P_data[ 'CC'   ]
        if( True == ( 'CXX'          in P_data ) ): self.m_native[ 'CXX'  ] = P_data[ 'CXX'  ]
        if( True == ( 'LINK'         in P_data ) ): self.m_native[ 'LINK' ] = P_data[ 'LINK' ]


    def get_native( self ):
        return self.m_native

    def get( self, P_name ):
        return self.m_native[ P_name ]

    def exists( self, P_name ):
        return True == ( P_name in self.m_native )

    def set( self, P_name, P_value ):
        self.m_native[P_name] = P_value

    def append( self, P_name, P_list ):
        #print( P_name + " + " + str(P_list) )
        self.m_native[P_name] += P_list

    ## Take all from other config by respecting P_param
    def accumulate( self, P_config, P_param = {} ):

        #print( 'Config::accumulate::P_config.m_native = ' + str( P_config.m_native ) )

        self._process_list( P_config.get_native(), 'CPPFLAGS'  , P_param )
        self._process_list( P_config.get_native(), 'CPPPATH'   , P_param )
        self._process_list( P_config.get_native(), 'CPPDEFINES', P_param )
        self._process_list( P_config.get_native(), 'LIBPATH'   , P_param )
        self._process_list( P_config.get_native(), 'LIBS'      , P_param )
        self._process_list( P_config.get_native(), 'LINKFLAGS' , P_param )

        self._process_value( P_config.get_native(), 'TARGET_ARCH' , P_param )
        self._process_value( P_config.get_native(), 'MSVC_VERSION' , P_param )

        self._process_value( P_config.get_native(), 'CC'   , P_param )
        self._process_value( P_config.get_native(), 'CXX'  , P_param )
        self._process_value( P_config.get_native(), 'LINK' , P_param )

        #print( 'Config::accumulate::self.m_native = ' + str( self.m_native ) )

    def _process_value( self, P_data, P_key, P_param={} ):
        if( False == ( P_key in P_data ) ):
            return;

        if( False ==  self.exists( P_key ) ):
            self.set( P_key, '' )

        if( True == hasattr( P_data[P_key], '__call__' ) ):
            self.set( P_key, P_data[P_key]( P_param ) );
            return

        self.set(  P_key, P_data[ P_key ] )

    def _process_list( self, P_data, P_key, P_param={} ):
        #print( P_data )
        if( False == ( P_key in P_data ) ):
            return;
        #print( P_key )

        if( False ==  self.exists( P_key ) ):
            self.set( P_key, [] )

        if( True == hasattr( P_data[P_key], '__call__' ) ):
            self.append( P_key, P_data[P_key]( P_param ) );
            return

        self.append( P_key, P_data[ P_key ] );



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import nucleotide
import nucleotide.component
import nucleotide.component.function
import nucleotide.component._common
import nucleotide.component._common.warning
import nucleotide.component._common.warning.util


def _linux_compiler_warning_CPPFLAGS( P_list ):
    Ir_list= []
    I_translator = nucleotide.translator.Translator( {'host'  : 'Linux','guest' : 'Linux'}, P_cc = { 'vendor': 'FSF', 'name': 'gcc', 'version': 'X'} )

    return nucleotide.component._common.warning.util.list( P_list, I_translator )

atom_linux_compiler_warning = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS' : _linux_compiler_warning_CPPFLAGS
    },
    'name' :'compiler:warning',
    'class':  [ 'compiler:warning', 'linux:compiler:warning' ]
}

class Warning:
    def __init__(self):
        pass

    @staticmethod
    def extend(P_option):
        nucleotide.component.function.extend( P_option, 'A:linux:compiler:warning',  atom_linux_compiler_warning )
        atom_linux_compiler_warning['platform']['host'] = 'X';
        nucleotide.component.function.extend( P_option, 'x:linux:compiler:warning',  atom_linux_compiler_warning )
        atom_linux_compiler_warning['platform']['guest'] = 'X';
        nucleotide.component.function.extend( P_option, 'y:linux:compiler:warning',  atom_linux_compiler_warning )

    @staticmethod
    def check(self):
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

import nucleotide.translator
import nucleotide.config
import nucleotide.klass
import nucleotide.component.windows._common.boost


def _boost_msvc12_CPPPATH( P_list ):
    I_arc = '32';
    #TODO

    I_result = os.getenv('BOOST_MSVC12_INCLUDE')
    if( None == I_result ):
        return nucleotide.component.windows._common.boost._boost_blank_CPPPATH( P_list )
    return [ I_result ]

def _boost_msvc12_LIBPATH( P_list ):
    I_arc = '32';
    I_result = os.getenv('BOOST_MSVC12_LIBPATH')
    if( None == I_result ):
        return nucleotide.component.windows._common.boost._boost_blank_LIBPATH( P_list )
    return [ I_result ]

atom_boost_msvc_12 = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor' : 'Microsoft',
        'name'   : 'msvc',
        'version': '12'
    },
    'config' : {
            'CPPPATH' : _boost_msvc12_CPPPATH,
            'LIBPATH' : _boost_msvc12_LIBPATH
    },
    'name' :'boost:msvc12',
    'class': [ 'boost', 'boost:blank', 'boost:msvc12'  ]
}



class Boost:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'boost:msvc12',          atom_boost_msvc_12          )

    def check():

        nucleotide.component.function.check__env( 'BOOST_MSVC12_INCLUDE' )
        nucleotide.component.function.check__env( 'BOOST_MSVC12_LIBPATH' )
        nucleotide.component.function.check__env( 'BOOST_MSVC12_VERSION' )
        nucleotide.component.function.check__env( 'BOOST_MSVC12_64_INCLUDE' )
        nucleotide.component.function.check__env( 'BOOST_MSVC12_64_LIBPATH' )
        nucleotide.component.function.check__env( 'BOOST_MSVC12_64_VERSION' )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


__all__       = [ 'Options', 'Settings', 'Atom', 'Environment', 'Config', 'Translator', 'Custom', 'component' ]
__name__      = 'nucleotide'
__author__    = 'I'
__developer__ = 'I'
__status__    = 'production'
__version__   = '0.0.0.0'
__date__      = '20:21 Friday, 02 October, 2015'
__revision__  = ''
__build__     = ''
__buildsys__  = ''


from  .config       import *
from  .translator   import *
from  .klass        import *
from  .custom       import *

from  .atom         import *

from  .options      import *

from  .catalog      import *

from  .settings     import *

from  .environment  import *

#from  .main        import *




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import nucleotide
import nucleotide.component
import nucleotide.component.function


def _linux_PDB_CPPFLAGS( P_data ):
    Ir_list = [ '-g' ]
    return Ir_list

    if( True == ( 'file-name-compile' in P_data ) ):
        Ir_list.append( 'TODO"' + P_data['file-name-compile'] + '"' )
    else:
        if( True == ( 'file-name' in P_data ) ):
            Ir_list.append( 'TODO"' + P_data['file-name'] + '"' )

    return Ir_list

atom_linux_PDB = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS'  : _linux_PDB_CPPFLAGS,
    },
    'name' : 'PDB',
    'class': [  'PDB', 'linux:PDB' ]
}

class PDB:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
         nucleotide.component.function.extend( P_option, 'A:linux:PDB',           atom_linux_PDB )
         atom_linux_PDB['platform']['host'] = 'X'; 
         nucleotide.component.function.extend( P_option, 'x:linux:PDB',           atom_linux_PDB )
         atom_linux_PDB['platform']['guest'] = 'X'; 
         nucleotide.component.function.extend( P_option, 'y:linux:PDB',           atom_linux_PDB )

    @staticmethod
    def check():
        pass
 



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import nucleotide
import nucleotide.component
import nucleotide.component.function

def __common_architecture_TARGET_ARCH( P_data ):
    if( False == ( 'name' in P_data ) ):
        return ''
    return P_data[ 'name' ]

atom_common_architecture = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor': 'X',
        'name': 'X',
        'version': 'X'
    },
    'config' : {
        'TARGET_ARCH' : __common_architecture_TARGET_ARCH
    },
    'name' :'architecture',
    'class':  [ 'architecture', 'compiler:architecture' ]
}

class Architecture:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'compiler:architecture',        atom_common_architecture )

    @staticmethod
    def check():
        pass




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _linux_Dialect_CPPFLAGS( P_data ):
    Ir_list = []

    if( True == ( 'dialect' in P_data ) ):
        if( 'plain_c' == P_data['dialect'] ):
            pass
        if( 'plain_cpp' == P_data['dialect'] ):
            pass

    if( True == ( 'standard' in P_data ) ):
        Ir_list += [ '-std=' + P_data['standard'] ]

    return Ir_list

atom_linux_gcc_dialect = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor': 'FSF',
        'name': 'gcc',
        'version': 'X'
    },
     'config' : {
        'CPPFLAGS'   : _linux_Dialect_CPPFLAGS
    },
    'name' :'source:c++:dialect',
    'class':  [ 'source:c++', 'source:c++:dialect' ,'linux:source:c++:dialect'  ]
}

class Dialect:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
        nucleotide.component.function.extend( P_option, 'A:linux:source:c++:dialect',   atom_linux_gcc_dialect )
        atom_linux_gcc_dialect['platform']['host'] = 'X'; 
        nucleotide.component.function.extend( P_option, 'x:linux:source:c++:dialect',   atom_linux_gcc_dialect )
        atom_linux_gcc_dialect['platform']['guest'] = 'X'; 
        nucleotide.component.function.extend( P_option, 'y:linux:source:c++:dialect',   atom_linux_gcc_dialect )

    @staticmethod
    def check():
        pass




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import re
import copy

## Translator infromation
#
class Translator:
    m_translator ={}

    def __init__( self, P_platform = {'host'  : 'X','guest' : 'X'}, P_cc = { 'vendor': 'X', 'name': 'X', 'version': 'X'} ):
        self.m_translator = {}
        self.m_translator['platform'] = copy.deepcopy( P_platform )
        self.m_translator['cc'] = copy.deepcopy( P_cc )

    def set_platform_host( self, P_value ):
        self.m_translator['platform']['host'] = P_value

    def set_platform_guest( self, P_value ):
        self.m_translator['platform']['guest'] = P_value

    def set_cc_vendor( self, P_value ):
        self.m_translator['cc']['vendor'] = P_value

    def set_cc_name( self, P_value ):
        self.m_translator['cc']['name'] = P_value

    def set_cc_version( self, P_value ):
        self.m_translator['cc']['version'] = P_value

    def get( self ):
        return self.m_translator

    def similarity( self, P_other ):
        I_level = -1
        #print( __file__ +' - P_other.m_translator == '+ str( P_other.m_translator ) )
        #print( __file__ +' - self.m_translator    == '+ str(    self.m_translator ) )

        I_match = Translator._similarity0( self.m_translator['platform']['host' ], P_other.m_translator['platform']['host' ] );
        if( 0 == I_match ):
            return -1

        I_level += 1

        I_match = Translator._similarity0( self.m_translator['platform']['guest' ], P_other.m_translator['platform']['guest' ] );
        if( 0 == I_match ):
            return -1

        I_level += 1

        I_match = Translator._similarity0( self.m_translator['cc']['vendor'], P_other.m_translator['cc']['vendor'] );
        I_level += [  -9000,    50,    50,    90 ][ I_match ]
        I_match = Translator._similarity0( self.m_translator['cc']['name'], P_other.m_translator['cc']['name'] );
        I_level += [  -9000,   500,   500,   900 ][ I_match ]
        I_match = Translator._similarity0( self.m_translator['cc']['version'], P_other.m_translator['cc']['version'] );
        I_level += [  -9000,  5000,  5000,  9000 ][ I_match ]

        return I_level


    ## Make blank information structure
    @staticmethod
    def blank():
        return copy.deepcopy( {
                'platform' : {
                    'host'  : 'X',
                    'guest' : 'X'
                },
                'cc' : {
                    'vendor' : 'X',
                    'name'   : 'X',
                    'version': 'X'
                }
            })


    @staticmethod
    def _similarity0( P_left, P_right ):
        #print( '        P_left = '+P_left+' , P_right = '+ P_right +' ' )
        if( P_left == P_right ):
            #print( '            2' )
            return 3
        if( 'X' == P_left ):
            #print( '            1' )
            return 2
        if( 'X' == P_right ):
            #print( '            1')
            return 1
        #print( '            0' )
        return 0;


    def string( self ):
        return \
                    self.m_translator['platform']['host'] \
            + '-' + self.m_translator['platform']['guest'] \
            + '-' + self.m_translator['cc']['vendor'] \
            + '-' + self.m_translator['cc']['name'] \
            + '-' + self.m_translator['cc']['version']


    @staticmethod
    def extract( P_string ):
        #print( 'Directory.extract(' + P_string + ')' )
        m = re.match( '(\w*)-(\w*)-(\w*)-(\w*)-(\w*)', P_string )
        if( None == m ):
            #print( 'Directory.extract:: return invalid' )
            return Translator( {'host'  : 'Y','guest' : 'Y'}, { 'vendor': 'Y', 'name': 'Y', 'version': 'Y'} )

        Ir_return = Translator( {'host' : m.group(1), 'guest' : m.group(2) }, { 'vendor': m.group(3), 'name': m.group(4), 'version': m.group(5)} )
        #print() 'Directory.extract( P_string =' + P_string + ' ) -> ' +str( Ir_return.m_translator ) )
        return Ir_return




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import subprocess

import nucleotide
import nucleotide.component
import nucleotide.component.function



def _python_linux_default_CPPFLAGS( P_list ):
    try:
        p = subprocess.Popen( [ '/usr/bin/python-config', ' --cflags' ],
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    except :
        return []
        
    output, error = p.communicate()
    if( None != error ):
        return []

    return output.split()

def _python_linux_default_LINKFLAGS( P_list ):
    try:
        p = subprocess.Popen( [ '/usr/bin/python-config', ' --ldflags' ],
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    except :
        return []

        output, error = p.communicate()
    if( None != error ):
        return []

    return output.split()

atom_python_linux_default = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
            'CPPFLAGS' : _python_linux_default_CPPFLAGS,
            'LINKFLAGS': _python_linux_default_LINKFLAGS,
    },
    'name' : 'python:linux:default',
    'class': [ 'python', 'python:default', 'python:linux:default' ]
}

def _python27_linux_CPPFLAGS( P_list ):
    try:
        p = subprocess.Popen( [ '/usr/bin/python2-config', ' --cflags' ],
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    except :
        return _python_linux_default_CPPFLAGS(P_list)

    output, error = p.communicate()
    if( None != error ):
        return []

    return output.split()

def _python27_linux_LINKFLAGS( P_list ):
    try:
        p = subprocess.Popen( [ '/usr/bin/python2-config', ' --ldflags'],
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    except :
        return _python_linux_default_LINKFLAGS( P_list )

    output, error = p.communicate()
    if( None != error ):
        return []

    return output.split()


atom_python27_linux= {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor': 'FSF',
        'name': 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS' : _python27_linux_CPPFLAGS,
        'LINKFLAGS': _python27_linux_LINKFLAGS,
    },
    'name' : 'python27:linux',
    'class':  [ 'python', 'python:default', 'python:linux:default', 'python27:default' , 'python27:linux' ]
}

def _python35_linux_CPPFLAGS( P_list ):
    try:
        p = subprocess.Popen( [ '/usr/bin/python3-config', ' --cflags' ],
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    except :
        return _python_linux_default_CPPFLAGS(P_list)

    output, error = p.communicate()
    if( None != error ):
        return []

    return output.split()

def _python35_linux_LINKFLAGS( P_list ):
    try:
        p = subprocess.Popen( [ '/usr/bin/python3-config', ' --ldflags' ],
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    except :
        return _python_linux_default_LINKFLAGS(P_list)

    output, error = p.communicate()
    if( None != error ):
        return []

    return output.split()


atom_python35_linux= {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor': 'FSF',
        'name': 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS' : _python35_linux_CPPFLAGS,
        'LINKFLAGS': _python35_linux_LINKFLAGS,
    },
    'name' : 'python35:linux',
    'class':  [ 'python', 'python:default', 'python:linux:default', 'python35:default', 'python35:linux' ]
}

class Python:
    def __init__(self):
        pass

    @staticmethod
    def extend(P_option):
        nucleotide.component.function.extend( P_option, 'python:linux:default',  atom_python_linux_default )
        nucleotide.component.function.extend( P_option, 'python27:linux',        atom_python27_linux )
        nucleotide.component.function.extend( P_option, 'python35:linux',        atom_python35_linux )

    def check(self):
        nucleotide.component.function.check__env( 'PYTHONPATH' )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform
import itertools
import copy
import sys


def find_ENVIRONMENT__generic( P_prefix, P_list, P_folder, P_ensure ):

    version =''
    architecture=''
    compiler=''

    I_element=[]
    if( "version"           in P_ensure ):
        if( True == ( 'version'         in P_list ) ): I_element += [ P_list['version']        ]
    if( "architecture"      in P_ensure ):
        if( True == ( 'architecture'    in P_list ) ): I_element += [ P_list["architecture"]   ]
    if( "compiler"          in P_ensure ):
        if( True == ( 'compiler'        in P_list ) ): I_element += [ P_list["compiler"]       ]
    if( "configuration"     in P_ensure ):
        if( True == ( 'configuration'   in P_list ) ): I_element += [ P_list["configuration"]  ]

    #print( " elements: " + str(I_element) )
    for prefix in P_prefix:
        for folder in P_folder:
            I_detail =  copy.deepcopy( I_element )
            if( "folder"     in P_ensure ):
                I_detail += [folder]
            #print( I_detail )
            #print( "    " + str(P_ensure) + str(I_detail) + prefix+" "+folder )

            #continue;
            for permutation in itertools.permutations( I_detail ):
                variation = [ prefix ]
                variation += permutation
                #print( variation )

                variable = ""
                for element in variation:
                    variable += element + "_"

                variable = variable.strip( "_" )
                #print( variable )
                e = os.environ.get( variable )
                if( None != e ):
                    #print( variable +" = "+ str( [ e ] ) )
                    return [ variable, e ]

    return []

def find_ENVIRONMENT_PATH( P_include, P_prefix, P_list ):

    I_decoration = [ "version", "architecture", "folder", "compiler", "configuration" ]

    max = 0;
    I_list = copy.deepcopy( P_list )
    I_list['version'] = ''

    if( False == "version"  in P_list ):
        P_list['version'] = ['']

    for flag in range( 0, 32 ):
        ornament = []
        if ( 0 != ( flag &  1 ) ): ornament += [ I_decoration[ 0 ]];
        if ( 0 != ( flag &  2 ) ): ornament += [ I_decoration[ 1 ]];
        if ( 0 != ( flag &  4 ) ): ornament += [ I_decoration[ 2 ]];
        if ( 0 != ( flag &  8 ) ): ornament += [ I_decoration[ 3 ]];
        if ( 0 != ( flag & 16 ) ): ornament += [ I_decoration[ 4 ]];

        #print( ornament )

        for version in P_list['version'] :
            I_list['version'] = version
            I_result = find_ENVIRONMENT__generic( P_prefix, I_list, P_include, ornament )
            if( 0 != len( I_result ) ):
                if( max < len( ornament ) ):
                    Ir_result = copy.deepcopy( I_result )
                    max = len( ornament )

    if( 0 == max ):
       print()
       return []
    print( Ir_result[0] + " = " +Ir_result[1] )
    return [ Ir_result[1] ]


def find_ENVIRONMENT_CPPPATH( P_prefix, P_list ):
    sys.stdout.write( '        CPPPATH: ' )

    I_include = { "INCLUDE", "INC",
                              "INCLUDE_PATH", "INC_PATH", "PATH_INC", "PATH_INCLUDE",
                              "INCLUDE_DIR",  "DIR_INCLUDE",
                              "INCLUDE_DIRS", "DIRS_INCLUDE",
                              "INCLUDEDIR", "DIRINCLUDE",
                              "PATH" }

    return find_ENVIRONMENT_PATH( I_include, P_prefix, P_list )


def find_ENVIRONMENT_LIBPATH( P_prefix, P_list ):
    sys.stdout.write('        LIBPATH: ' )

    I_library ={ "LIBRARY", "LIB", "LIBS", "LIBRARIES", "LIBPATH",
                          "LIBRARYPATH",  "LIBRARY_PATH", "LIB_PATH", "LIBS_PATH", "PATH_LIBS", "PATH_LIB",
                          "LIBRARYDIR",   "LIBRARY_DIR",  "LIB_DIR",  "LIBS_DIR",  "DIR_LIBS",  "DIR_LIB",
                          "LIBRARYDIRS",  "LIBRARY_DIRS", "LIB_DIRS", "LIBS_DIRS", "DIRS_LIBS", "DIRS_LIB",
                          "PATH" }

    return find_ENVIRONMENT_PATH( I_library, P_prefix, P_list )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


#__all__   = [ 'table', 'util' ]
#__name__  = 'nucleotide.component.warning'

#import main


#print( 'End importing Module: -|' + __name__ )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import subprocess
import platform
import glob
import re

import nucleotide
import nucleotide.component
import nucleotide.component.linux
import nucleotide.component.linux.gcc
import nucleotide.component.linux.gcc.atom
import nucleotide.component.linux.gcc.atom._misc
import nucleotide.component.linux.gcc.atom.python
import nucleotide.component.linux.gcc.atom.warning
import nucleotide.component.linux.gcc.atom.pdb
import nucleotide.component.linux.gcc.atom.blank
import nucleotide.component.linux.gcc.atom.rtl
import nucleotide.component.linux.gcc.atom.rtti
import nucleotide.component.linux.gcc.atom.pp2f
import nucleotide.component.linux.gcc.atom.exception
import nucleotide.component.linux.gcc.atom.dialect
import nucleotide.component.linux.gcc.atom.encode
import nucleotide.component.linux.gcc.atom.executable
import nucleotide.component.linux.gcc.atom.macro
import nucleotide.component.linux.gcc.atom.version
import nucleotide.component.linux.gcc.atom.configuration
import nucleotide.component.linux.gcc.atom.optimization
import nucleotide.component.linux.gcc.atom.package
import nucleotide.component.linux.gcc.atom.alignment

## Detect GCC on linux
class Translator:
    m_list = []

    def __init__(self):
        self.m_list = []

        I_data = {
                'platform' : {
                    'host'  : 'Linux',
                    'guest' : 'Linux'
                },
                'cc' : {
                    'vendor' : 'FSF',
                    'name'   : 'gcc',
                    'version': 'X'
                }
            }
        self.m_list += [ nucleotide.Translator( I_data['platform'], I_data['cc'] ) ]

        r = re.compile('/usr/bin/gcc-(([0-9]+)(\\.[0-9]+)*)')

        list = glob.glob("/usr/bin/gcc-*")

        for item in list:
            m = r.search( item )
            if( None == m ):
                continue
            I_data = {
                    'platform' : {
                        'host'  : 'Linux',
                        'guest' : 'Linux'
                    },
                    'cc' : {
                        'vendor' : 'FSF',
                        'name'   : 'gcc',
                        'version': m.group(1)
                    }
                }
            self.m_list += [ nucleotide.Translator( I_data['platform'], I_data['cc'] ) ]

    def get(self):
        return self.m_list

    @staticmethod
    def check():
        return True

    @staticmethod
    def extend(P_options):
        nucleotide.component.linux.gcc.atom._misc.init(P_options)
        nucleotide.component.linux.gcc.atom.python.Python.extend(P_options)
        nucleotide.component.linux.gcc.atom.warning.Warning.extend(P_options)
        nucleotide.component.linux.gcc.atom.pdb.PDB.extend(P_options)
        nucleotide.component.linux.gcc.atom.blank.Blank.extend(P_options)
        nucleotide.component.linux.gcc.atom.rtl.RTL.extend(P_options)
        nucleotide.component.linux.gcc.atom.rtti.RTTI.extend(P_options)
        nucleotide.component.linux.gcc.atom.pp2f.PP2F.extend(P_options)
        nucleotide.component.linux.gcc.atom.exception.Exception.extend(P_options)
        nucleotide.component.linux.gcc.atom.dialect.Dialect.extend(P_options)
        nucleotide.component.linux.gcc.atom.encode.Encode.extend(P_options)
        nucleotide.component.linux.gcc.atom.executable.Executable.extend(P_options)
        nucleotide.component.linux.gcc.atom.macro.Macro.extend(P_options)
        nucleotide.component.linux.gcc.atom.version.Version.extend(P_options)
        nucleotide.component.linux.gcc.atom.configuration.Configuration.extend(P_options)
        nucleotide.component.linux.gcc.atom.optimization.Optimization.extend(P_options)
        nucleotide.component.linux.gcc.atom.package.Package.extend(P_options)
        nucleotide.component.linux.gcc.atom.alignment.Alignment.extend(P_options)





#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _common_library_LIBPATH( P_list ):

    Ir_list = []

    for item in P_list:
        if( False == ( 'path' in P_list[item] ) ):
            continue
        Ir_list += [ P_list[item]['path'] ]

    #print( 'Library::_common_library_LIBPATH::Ir_list =  ' + str( Ir_list ) )

    return Ir_list

def _common_library_LIBS( P_list ):

    Ir_list = []

    for item in P_list:
        if( False == ( 'list' in P_list[item] ) ):
            continue
        Ir_list += P_list[item]['list']

    #print( 'Library::_common_library_LIBS::Ir_list =  ' + str( Ir_list ) )

    return Ir_list


atom__common_library = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor' : 'X',
        'name'   : 'X',
        'version': 'X'
    },
    'config' : {
        'LIBPATH' : _common_library_LIBPATH,
        'LIBS'    : _common_library_LIBS,
    },
    'name' :'library',
    'class':  [ 'library' ]
}

class Library:

    def __init( self ) :
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'library',    atom__common_library )

    @staticmethod
    def check():
        pass


#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import subprocess

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _linux_gcc_version_GCC_CC_VERSION( P_data ):
    if( False == ( 'gcc' in P_data ) ):
        return None
    return 'gcc-' + P_data[ 'gcc' ][0]

def _linux_gcc_version_GCC_CXX_VERSION( P_data ):
    if( False == ( 'gcc' in P_data ) ):
        return None
    return 'g++-' + P_data[ 'gcc' ][0]

def _linux_gcc_version_GCC_LINK_VERSION( P_data ):
    if( False == ( 'gcc' in P_data ) ):
        return None
    return 'g++-' + P_data[ 'gcc' ][0]


atom_linux_CCVERSION = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor': 'FSF',
        'name': 'gcc',
        'version': 'X'
    },
     'config' : {
        'CC'    : _linux_gcc_version_GCC_CC_VERSION,
        'CXX'   : _linux_gcc_version_GCC_CXX_VERSION,
        'LINK'  : _linux_gcc_version_GCC_LINK_VERSION
    },
    'name' :'compiler:version',
    'class':  [ 'compiler:version', 'linux:compiler:version' ]
}

class Version:
    def __init__(self):
        pass

    @staticmethod
    def extend(P_option):
        nucleotide.component.function.extend( P_option, 'linux:compiler:version', atom_linux_CCVERSION )

    @staticmethod
    def check(self):
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import nucleotide
import nucleotide.component
import nucleotide.component.function

def _atom_windows_msvc_optimization_CPPFLAGS( P_data ):
    Ir_list = []
    return Ir_list


def _atom_windows_msvc_optimization_CPPDEFINES( P_data ):
    Ir_list = []
    return Ir_list

def _atom_windows_msvc_optimization_LINKFLAGS( P_data ):
    Ir_list = []
    return Ir_list

def _atom_windows_msvc_optimization_LIBS( P_data ):
    Ir_list = []
    return Ir_list


atom_windows_msvc_optimization = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor': 'Microsoft',
        'name': 'msvc',
        'version': 'X'
    },
    'name' :'optimization',
    'config' : {
        'CPPFLAGS'    : _atom_windows_msvc_optimization_CPPFLAGS,
        'CPPDEFINES'  : _atom_windows_msvc_optimization_CPPDEFINES,
        'LINKFLAGS'   : _atom_windows_msvc_optimization_LINKFLAGS,
        'LIBS'        : _atom_windows_msvc_optimization_LIBS
    },
    'class': [ 'optimization', 'compiler:optimization', 'windows:compiler:optimization',
               'optimisation', 'compiler:optimisation', 'windows:compiler:optimisation'
    ]
}

class Optimization:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
         nucleotide.component.function.extend( P_option, 'A:windows:optimization',               atom_windows_msvc_optimization)
         nucleotide.component.function.extend( P_option, 'A:windows:optimisation',               atom_windows_msvc_optimization)

    @staticmethod
    def check():
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import nucleotide
import nucleotide.component
import nucleotide.component.function
import nucleotide.component._common

import nucleotide.translator
import nucleotide.config
import nucleotide.klass

atom_linux_shared_library = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        # Nothing. It is empty
    },
    'name' :'dll',
    'class':  [ 'executable:shared_object', 'linux:executable:shared_object' ]
}

atom_linux_shared_object = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor': 'FSF',
        'name': 'gcc',
        'version': 'X'
    },
     'config' : {
           'CPPDEFINES'  : ['_LIB' ],
    },
    'name' :'lib',
    'class': [  'shared:library' ]
}

def init( P_option ) :

   #nucleotide.component.function.extend( P_option, 'architecture',        atom_linux_architecture )

    nucleotide.component.function.extend( P_option, 'linux:shared_library',        atom_linux_shared_library)
    nucleotide.component.function.extend( P_option, 'linux:shared_object',         atom_linux_shared_object)
    #nucleotide.component.function.extend( P_option, 'linux:static_release',        atom_linux_static_release)

   #nucleotide.component.function.extend( P_option, 'linux:linker:warning',    atom_linux_linker_warning )
   #nucleotide.component.function.extend( P_option, 'linux:compiler:feature:for-scope',  atom_linux_compiler_warning )   -ffor-scope







#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import nucleotide

## Minimal action thats configure translator
class Atom:
    m_translator = None
    m_config    = None #!< Native configuration. In form (key,data|function)
    m_klass     = None

    def __init__( self, P_translator = None, P_config = None, P_class = None ):

        if( None == P_translator ):
            P_translator = nucleotide.translator.Translator()
        self.m_translator = P_translator

        if( None == P_config ):
            P_config = nucleotide.config.Config()
        self.m_config    = P_config

        if( None == P_class ):
            P_class = nucleotide.klass.Klass()
        self.m_klass     = P_class

    def set_translator( self, P_value ):
        self.m_translator = P_value

    def get_translator( self ):
        return self.m_translator

    def get_config( self ):
        return self.m_config

    def set_config( self, P_value ):
        self.m_config = P_value

    def get_klass( self ):
        return self.m_klass

    def set_klass( self, P_value ):
        self.m_klass = P_value





#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

def atom_linux_configuration_CPPFLAGS( P_data ):
    Ir_list = []

    if( False == ( 'name' in P_data ) ):
        return Ir_list

    if( 'debug' == P_data[ 'name' ].lower() ):
        Ir_list += [ '-g'  ]

    if( 'release'== P_data[ 'name' ].lower() ):
        pass

    return Ir_list

def atom_linux_configuration_CPPDEFINES( P_data ):
    Ir_list = []

    if( False == ( 'name' in P_data ) ):
        return Ir_list

    if( 'debug' == P_data[ 'name' ].lower() ):
        Ir_list += [ '_DEBUG'  ]

    if( 'release' == P_data[ 'name' ].lower() ):
        Ir_list += [ 'NDEBUG'  ]

    return Ir_list

def atom_linux_configuration_LINKFLAGS( P_data ):
    Ir_list = []

    if( False == ( 'name' in P_data ) ):
        return Ir_list

    if( 'debug' == P_data[ 'name' ].lower() ):
        pass #Ir_list += [ 'TODO'  ]

    if( 'release'== P_data[ 'name' ].lower() ):
        pass

    return Ir_list


atom_linux_configuration = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS'   : atom_linux_configuration_CPPFLAGS,
        'CPPDEFINES' : atom_linux_configuration_CPPDEFINES,
        'LINKFLAGS'  : atom_linux_configuration_LINKFLAGS
    },
    'name' :'compiler:configuration',
    'class': [ 'compiler:configuration', 'linux:compiler:configuration' ]
}

class Configuration:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
         nucleotide.component.function.extend( P_option, 'A:linux:compiler:configuration',               atom_linux_configuration )
         atom_linux_configuration['platform']['host']='X'
         nucleotide.component.function.extend( P_option, 'x:linux:compiler:configuration',               atom_linux_configuration )
         atom_linux_configuration['platform']['guest']='X'
         nucleotide.component.function.extend( P_option, 'y:linux:compiler:configuration',               atom_linux_configuration )

    @staticmethod
    def check():
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import subprocess
import platform
import os

import nucleotide
import nucleotide.translator
import nucleotide.component
import nucleotide.component.windows
import nucleotide.component.windows.mingw
import nucleotide.component.windows.mingw.atom
import nucleotide.component.windows.mingw.atom.python

## Detect MinGW on Windows
class Translator:
    m_list = []
    def __init__(self):
        self.m_list = []

        try:
            process = subprocess.Popen( [ 'gcc', '-dumpversion' ], stdout = subprocess.PIPE )
        except( EnvironmentError ):
            return

        version = str.split( process.communicate()[0], os.linesep )[0]
        self.m_list.append( nucleotide.Translator(
                {
                    'host'  : 'Windows',
                    'guest' : 'Windows'
                },
                {
                    'vendor' : 'FSF',
                    'name'   : 'gcc',
                    'version': version
                }
              ) )

    def get(self):
        return self.m_list

    @staticmethod
    def check(self):
        pass

    @staticmethod
    def extend(P_options):
        nucleotide.component.windows.mingw.atom.python.Python.extend(P_options)





#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import subprocess

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _windows_RTL_CPPFLAGS( P_data ):
    I_flag = 'M'

    if( 'dynamic' == P_data['type'].lower() ):
       I_flag += 'D'
    if( 'static' == P_data['type'].lower() ):
       I_flag += 'T'

    if( True == ( 'configuration' in P_data ) ):
        if( 'debug' == P_data['configuration'].lower() ):
           I_flag += 'd'
        if( 'release' == P_data['configuration'].lower() ):
           pass

    return [ '/' + I_flag ]

atom_windows_RTL= {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor': 'Microsoft',
        'name': 'msvc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS'  : _windows_RTL_CPPFLAGS
    },
    'name' :'RTL',
    'class':  [ 'RTL', 'windows:RTL' ]
}

class RTL:
    def __init__(self):
        pass

    @staticmethod
    def extend(P_option):
        nucleotide.component.function.extend( P_option, 'windows:RTL',  atom_windows_RTL )

    @staticmethod
    def check(self):
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

# type = [ 'console', 'UI' ]

def _linux_executable_CPPFLAGS( P_data ):
    Ir_list= [] # TODO
    return Ir_list

def _linux_executable_CPPDEFINES( P_data ):
    Ir_list= []# TODO
    return Ir_list

def _linux_executable_LIBS( P_data ):
    Ir_list= []# TODO
    return Ir_list

def _linux_executable_LINKFLAGS( P_data ):
    Ir_list= []# TODO
    return Ir_list

atom_linux_gcc_executable = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor': 'FSF',
        'name': 'gcc',
        'version': 'X'
    },
     'config' : {
        'CPPFLAGS'    : _linux_executable_CPPFLAGS,
        'CPPDEFINES'  : _linux_executable_CPPDEFINES,
        'LIBS'        : _linux_executable_LIBS,
        'LINKFLAGS'   : _linux_executable_LINKFLAGS
    },
    'name' :'linux:executable',
    'class':  [ 'executable', 'linux:executable' ]
}

class Executable:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
        nucleotide.component.function.extend( P_option, 'linux:executable',         atom_linux_gcc_executable)

    @staticmethod
    def check():
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

## TODO join this with 'boost'
atom_boost_thread = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        'LINKFLAGS'  : [ '-pthread', '-lboost_thread', '-lboost_system' ],
        'CPPFLAGS'   : [ '-fopenmp' ],
        'CPPDEFINES' : [ 'BOOST_LOG_DYN_LINK' ]
    },
    'name' : 'boost:thread',
    'class':  [ 'boost:thread' ]
}

class Boost:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
         nucleotide.component.function.extend( P_option, 'boost:blank',           atom_boost_thread )

    @staticmethod
    def check():
        pass




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import nucleotide
import nucleotide.component
import nucleotide.component.function


component_windows_cygwingcc_atom_blank = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'cygwin'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'cygwingcc',
        'version': 'X' 
    },
    'name' :'blank',
    'config' : {
        'CPPDEFINES'   : [ ],
        'CPPFLAGS'     : [],
        'LIBS'   : [ ]
    },
    'class': [ 'blank', 'cygwingcc:blank' ]
}

class Blank:
    def __init__(self):
        pass

    @staticmethod
    def extend(P_option):
        nucleotide.component.function.extend( P_option, 'cygwingcc:blank', component_windows_cygwingcc_atom_blank )

    @staticmethod
    def check(self):
        pass
 


#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import platform

import nucleotide
import nucleotide.component
import nucleotide.component.windows
import nucleotide.component.windows.cygwingcc.atom
import nucleotide.component.windows.cygwingcc.atom.blank


##Detect GCC on CYgWin
class Translator:
    m_list = []
    def __init__(self):
        self.m_list = []

    def get(self):
        return self.m_list

    @staticmethod
    def check():
        return True

    @staticmethod
    def extend(P_options):
        nucleotide.component.windows.cygwingcc.atom.blank.Blank.extend(P_options) 
        nucleotide.component.linux.gcc.translator.Translator.extend(P_options) 

    @staticmethod
    def _exists( key, sub_key ):
        return False



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import nucleotide


## Manipulate with all data needed to generating Scons.Environment
# Most important function is accumulate. Next one is extend.
class Settings:
    m_translator = None
    m_options    = None # Option or what is possible
    m_config     = None # This goes to Environment
    m_custom     = None
    m_info       = None

    def __init__( self, P_translator = None, P_options = None, P_custom = None ):
        if( None != P_translator ):
            self.m_translator = P_translator;
        else:
            catalog = nucleotide.Catalog()
            if( 0 != len( catalog.get() ) ):
                self.m_translator = catalog.get()[0]

        self.m_config = nucleotide.Config()
        if( None == P_options ):
            P_options = nucleotide.Options( True )
        self.m_options = P_options

        if( None == P_custom ):
            P_custom = nucleotide.Custom()
        self.m_custom = P_custom

    def get_translator( self ):
        return self.m_translator

    def get_options( self ):
        return self.m_options

    def get_config( self ):
        return self.m_config

    def exists_custom( self, P_name ):
        return self.m_custom.exists( P_name )

    def get_custom( self, P_name ):
        return self.m_custom.get( P_name )

    def set_custom( self, P_name, P_value ):
        self.m_custom.set( P_name, P_value )

    def get_info( self ):
        return self.m_info

    ##Add specified atom ( as unified name ) to be available for accumulating
    def extend( self, P_name, P_atom ):
        self.m_options.extend( P_name, P_atom )

    ##Add specified atom ( as unified name ) so the latter will be used for creating Scons.environment
    def accumulate( self, P_name, P_parameter = {} ):
        I_atom = self.m_options.get( self.m_translator, P_name );

        self.m_config.accumulate( I_atom.get_config(), P_parameter )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import SCons.Script

## Create Scons Environment from accumulated atoms from the given settings
class Environment:

    M_native = None

    def __init__( self, P_settings ):

        #print( 'Environment::P_settings.get_config().m_native: ' + str( P_settings.get_config().m_native ) )

        I_init = {}

        if( True == P_settings.get_config().exists( 'TARGET_ARCH' ) ):
            #print( 'TARGET_ARCH: -|' + str( P_settings.get_config().get( 'TARGET_ARCH'  ) ) + '|-' )
            I_init[ 'TARGET_ARCH' ] = P_settings.get_config().get( 'TARGET_ARCH' )

        if( True == P_settings.get_config().exists( 'MSVC_VERSION' ) ):
            if( None != P_settings.get_config().get( 'MSVC_VERSION' ) ):
            #print( 'MSVC_VERSION: -|' + str( P_settings.get_config().get( 'MSVC_VERSION'  ) ) + '|-' )
                I_init[ 'MSVC_VERSION' ] = P_settings.get_config().get( 'MSVC_VERSION' )

        if( True == P_settings.get_config().exists( 'tools' ) ):
            if( None != P_settings.get_config().get( 'tools' ) ):
            #print( 'MSVC_VERSION: -|' + str( P_settings.get_config().get( 'MSVC_VERSION'  ) ) + '|-' )
                I_init[ 'tools' ] = P_settings.get_config().get( 'tools' )

        self.M_native = SCons.Script.Environment( **I_init )

        if( P_settings.get_config().exists( 'CPPFLAGS'   ) ): self.M_native.Append( CPPFLAGS    = P_settings.get_config().get( 'CPPFLAGS'  ) )
        if( P_settings.get_config().exists( 'CPPPATH'    ) ): self.M_native.Append( CPPPATH     = P_settings.get_config().get( 'CPPPATH'   ) )
        if( P_settings.get_config().exists( 'CPPDEFINES' ) ): self.M_native.Append( CPPDEFINES  = P_settings.get_config().get( 'CPPDEFINES') )
        if( P_settings.get_config().exists( 'LIBPATH'    ) ): self.M_native.Append( LIBPATH     = P_settings.get_config().get( 'LIBPATH'   ) )
        if( P_settings.get_config().exists( 'LIBS'       ) ): self.M_native.Append( LIBS        = P_settings.get_config().get( 'LIBS'      ) )
        if( P_settings.get_config().exists( 'LINKFLAGS'  ) ): self.M_native.Append( LINKFLAGS   = P_settings.get_config().get( 'LINKFLAGS' ) )

        if( True == P_settings.get_config().exists( 'CC'   ) ) and ( None != P_settings.get_config().get( 'CC'   ) ): self.M_native.Replace( CC   = P_settings.get_config().get( 'CC'   ) )
        if( True == P_settings.get_config().exists( 'CXX'  ) ) and ( None != P_settings.get_config().get( 'CXX'  ) ): self.M_native.Replace( CXX  = P_settings.get_config().get( 'CXX'  ) )
        if( True == P_settings.get_config().exists( 'LINK' ) ) and ( None != P_settings.get_config().get( 'LINK' ) ): self.M_native.Replace( LINK = P_settings.get_config().get( 'LINK' ) )

        #print( P_settings.get_config().get( 'CPPFLAGS'  )  )
        #print( P_settings.get_config().get( 'CPPPATH'  )   )
        #print( P_settings.get_config().get( 'CPPDEFINES'   )
        #print( P_settings.get_config().get( 'LIBPATH'  )   )
        #print( P_settings.get_config().get( 'LIBS'  )      )
        #print( P_settings.get_config().get( 'LINKFLAGS'  ) )

    def native( self ):
        return self.M_native



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import nucleotide
import nucleotide.component
import nucleotide.component.function


def _linux_Macro_CPPFLAGS( P_data ):
    Ir_list = []

    if( False== ( 'name' in P_data ) ):
        return Ir_list

    s = '-D' + P_data[ 'name' ]

    if( True == ( 'parameter' in P_data ) ):
        s += '('
        for p in P_data['parameter']:
            s += p
            s += ','
        s = s = s[:-1]
        s += ')'

    if( True == ( 'body' in P_data ) ):
        s += '=' + P_data[ 'body' ]

    Ir_list += [ s ]
    return Ir_list

atom_linux_Macro = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS'  : _linux_Macro_CPPFLAGS,
    },
    'name' : 'macro',
    'class': [  'macro', 'linux:macro' ]
}

# { 'name' : 'MY_MAX', 'parameter':['a', 'b'], 'body': '( (a)<(b) ? (b) : (a) )' }
class Macro:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
        nucleotide.component.function.extend( P_option, 'A:linux:compiler:macro',  atom_linux_Macro )
        atom_linux_Macro['platform']['host'] = 'X';
        nucleotide.component.function.extend( P_option, 'x:linux:compiler:macro',  atom_linux_Macro )
        atom_linux_Macro['platform']['guest'] = 'X';
        nucleotide.component.function.extend( P_option, 'y:linux:compiler:macro',  atom_linux_Macro )

    @staticmethod
    def check():
        pass




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import nucleotide
import nucleotide.component
import nucleotide.component.function

def _windows_configuration_CPPDEFINES( P_data ):
    Ir_list = []

    if( False == ( 'name' in P_data ) ):
        return Ir_list

    if( 'debug' == P_data[ 'name' ].lower() ):
        Ir_list += [ '_DEBUG' ]

    if( 'release'== P_data[ 'name' ].lower() ):
        Ir_list += [ 'NDEBUG' ]
    return Ir_list

def _windows_configuration_CPPFLAGS( P_data ):
    Ir_list = []

    if( False == ( 'name' in P_data ) ):
        return Ir_list

    if( 'debug' == P_data[ 'name' ].lower() ):
        Ir_list += [ '/RTC1'  ]

    if( 'release'== P_data[ 'name' ].lower() ):
        pass

    return Ir_list

def _windows_configuration_LINKFLAGS( P_data ):
    Ir_list = []

    if( False == ( 'name' in P_data ) ):
        return Ir_list

    if( 'debug' == P_data[ 'name' ].lower() ):
        Ir_list += [ '/DEBUG' ]

    if( 'release'== P_data[ 'name' ].lower() ):
        pass

    return Ir_list

# debug   'CPPDEFINES' : [ '_DEBUG' ],
# debug   'CPPFLAGS'   : [ '/RTC1'  ],
# debug   'LINKFLAGS'  : [ '/DEBUG' ]

# release:  'CPPDEFINES'   : [ 'NDEBUG' ],

atom_windows_configuration = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor': 'Microsoft',
        'name': 'msvc',
        'version': 'X'
    },
    'config' : {
        'CPPDEFINES' : _windows_configuration_CPPDEFINES,
        'CPPFLAGS'   : _windows_configuration_CPPFLAGS,
        'LINKFLAGS'  : _windows_configuration_LINKFLAGS
    },
    'name' :'compiler:configuration',
    'class': [ 'compiler:configuration', 'windows:compiler:configuration' ]
}

atom_windows_shared_library = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor': 'Microsoft',
        'name': 'msvc',
        'version': 'X'
    },
    'config' : {
        'CPPDEFINES'  : ['_USRDLL', '_WINDLL' ],
        'LINKFLAGS'   : [ '/DLL', '/SUBSYSTEM:WINDOWS' ]
    },
    'name' :'dll',
    'class':  [ 'shared:library' ]
}

atom_windows_shared_object = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor': 'Microsoft',
        'name': 'msvc',
        'version': 'X'
    },
     'config' : {
        'CPPDEFINES'  : ['_LIB' ],
    },
    'name' :'lib',
    'class': [  'shared:object' ]
}


def _windows_optimization_CPPFLAGS( P_data ):
    Ir_list = []
    if( True == ( 'configuration' in P_data ) ):
        if( 'debug' == P_data['configuration'].lower() ):
           Ir_list.append( '/Od' )
        if( 'release' == P_data['configuration'].lower() ):
           Ir_list.append( '/O2' )

    return Ir_list

atom_windows_optimization = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor': 'Microsoft',
        'name': 'msvc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS'  : _windows_optimization_CPPFLAGS,
    },
    'name' : 'release-optimization',
    'class': [ 'compiler:optimization', 'windows:compiler:optimization' ]
}


def init( P_option ) :

    nucleotide.component.function.extend( P_option, 'windows:compiler:configuration',    atom_windows_configuration )
    nucleotide.component.function.extend( P_option, 'windows:compiler:optimization',     atom_windows_optimization )

    nucleotide.component.function.extend( P_option, 'windows:shared:library', atom_windows_shared_library)
    nucleotide.component.function.extend( P_option, 'windows:shared:object',  atom_windows_shared_object)

   #nucleotide.component.function.extend( P_option, 'windows:linker:warning',      atom_windows_linker_warning )
   #nucleotide.component.function.extend( P_option, 'windows:linker:optimization', atom_windows_linker_warning )




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _linux_RTTI_CPPFLAGS( P_data ):
    if( False == ( 'enable' in P_data ) ):
        return [ '' ]

    if( 'true' == P_data[ 'enable' ] ):
        return [ '' ]

    if( 'false' == P_data[ 'enable' ] ):
        return [ '-fno-rtti' ]

    return [ '' ]

atom_linux_RTTI = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
     'config' : {
        'CPPFLAGS'   : _linux_RTTI_CPPFLAGS
    },
    'name' :'RTTI',
    'class':  [ 'RTTI', 'linux:RTTI' ]
}

class RTTI:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
         nucleotide.component.function.extend( P_option, 'A:linux:RTTI',                atom_linux_RTTI)
         atom_linux_RTTI['platform']['host'] = 'X';
         nucleotide.component.function.extend( P_option, 'x:linux:RTTI',                atom_linux_RTTI)
         atom_linux_RTTI['platform']['guest'] = 'X';
         nucleotide.component.function.extend( P_option, 'y:linux:RTTI',                atom_linux_RTTI)

    @staticmethod
    def check():
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import subprocess
import platform
import os

import nucleotide
import nucleotide.component
import nucleotide.component.linux
import nucleotide.component.linux._common
import nucleotide.component.linux._common.translator
import nucleotide.component.linux.gcc
import nucleotide.component.linux.gcc.translator

## Detect Translators on Linux
class Translator:
    m_list = []
    def __init__(self):
        self.m_list = []
        if( 'Linux' != platform.system() ):
            return

        I__common  = nucleotide.component.linux._common.translator.Translator()
        self.m_list += I__common.get()

        I_gcc      = nucleotide.component.linux.gcc.translator.Translator()
        self.m_list += I_gcc.get()

    def get(self):
        return self.m_list

    def check(self):
        pass

    @staticmethod
    def extend(P_options):
        if( 'Linux' != platform.system() ):
            return

        nucleotide.component.linux._common.translator.Translator.extend(P_options)
        nucleotide.component.linux.gcc.translator.Translator.extend(P_options)




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

import nucleotide.translator
import nucleotide.config
import nucleotide.klass

def _boost_mingw_CPPPATH( P_list ):
    I_result = os.getenv('BOOST_MINGW_INCLUDE')
    if( None == I_result ):
        return _boost_blank_CPPPATH( P_list )
    return [ I_result ]

def _boost_mingw_LIBPATH( P_list ):
    I_result = os.getenv('BOOST_MINGW_LIBPATH')
    if( None == I_result ):
        return _boost_blank_LIBPATH( P_list )
    return [ I_result ]

atom_boost_mingw = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor' : 'mingw',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
            'CPPPATH' : _boost_mingw_CPPPATH,
            'LIBPATH' : _boost_mingw_LIBPATH
    },
    'name' :'boost:mingw',
    'class': [ 'boost', 'boost:blank', 'boost:mingw'  ]
}


class Boost:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ) :
            nucleotide.component.function.extend( P_option, 'boost:mingw',           atom_boost_mingw            )

    @staticmethod
    def check():

        function.check__env( 'BOOST_MINGW_INCLUDE' )
        function.check__env( 'BOOST_MINGW_LIBPATH' )
        function.check__env( 'BOOST_MINGW_VERSION' )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os

import nucleotide
import nucleotide.component.constant
#TODO import nucleotide.translator.Translator
#TODO import nucleotide.config.Config
#TODO import nucleotide.klass.Klass
#TODO import nucleotide.atom.Atom



def join_out2(a,b):
    return a + separator_out  + b

def join_in2(a,b):
    return a + separator_in  + b

def join_in4(a,b,c,d):
    return a + separator_in  + b+ separator_in  + c + separator_in  + d

def join_in6(a,b,c,d,e):
    return a + separator_in  + b+ separator_in  + c + separator_in  + d + separator_in  + e

def prefix( P_atom ):
    return join_in4( P_atom['platform']['host'], P_atom['platform']['guest'],P_atom['cc']['vendor'], P_atom['cc']['name'], P_atom['cc']['version'] )

def make_name( P_atom ):
    return join_out2( prefix( P_atom ), P_atom['name'] )

def append( P_option, P_class, P_data ):
    full_name = make_name( P_data );
    P_option[ full_name ] = P_data['config']
    P_class[ P_data['class'] ]  += [ full_name ]

def extend( P_option, P_name, P_data ):
    I_translator = nucleotide.translator.Translator( P_data['platform'], P_data['cc'] )
    I_config    = nucleotide.config.Config( P_data['config']  )
    I_klass     = nucleotide.klass.Klass( P_data['class'] )

    I_atom = nucleotide.atom.Atom( I_translator, I_config, I_klass )
    P_option.extend( P_name, I_atom )


def check__env( P_name ) :
    value = os.getenv( P_name )
    if None == value:
        print(  P_name + ' Not exists' )
        return False
    print( P_name + ': -|' + value +'|-' )
    return True




from setuptools import find_packages
from distutils.core import setup
import time

import os
import glob

rootdir = os.path.abspath( os.path.dirname(__file__) )
print ( rootdir )

#my_version = "0.1.0" +  str(time.time()),
my_version = "0.1.4.8"

my_packages = find_packages( rootdir )

with open( os.path.join( rootdir, "readme.txt" ), "r") as fh:
    my_long_description = fh.read()

print ( my_long_description )
print ( my_packages )

setup(
    name="nucleotide",
    version          = my_version,
    keywords         = 'scons, module, universal, unified, compiler, c++',
    author           = "Dejan Milosavljevic",
    author_email     = "dmilos@gmail.com",
    url              = "https://github.com/dmilos/nucleotide",
    license          = "https://raw.githubusercontent.com/dmilos/nucleotide/master/license.txt",
    packages         = my_packages,
    description      = "An Scons library which wraps interface of various compilers in to one universal.",
    long_description = my_long_description,
    platforms        = ["All"],
    #install_requires = [ "scons"  ],
    # NO scripts = [ "nucleotide"  ]
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Software Development :: Build Tools'
    ],
)



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import subprocess
import platform
import os

import nucleotide
import nucleotide.component
import nucleotide.component.windows
import nucleotide.component.windows._common
import nucleotide.component.windows._common.translator
import nucleotide.component.windows.mingw
import nucleotide.component.windows.mingw.translator
import nucleotide.component.windows.msvc
import nucleotide.component.windows.msvc.translator
import nucleotide.component.windows.cygwingcc
import nucleotide.component.windows.cygwingcc.translator

## Detect MinGW on Windows
class Translator:
    m_list = []
    def __init__(self):
        self.m_list = []

        if( False == Translator._detect() ):
            return

        I__common = nucleotide.component.windows._common.translator.Translator()
        self.m_list += I__common.get()

        I_mingw    = nucleotide.component.windows.mingw.translator.Translator()
        self.m_list += I_mingw.get()

        if( 'Windows' == platform.system() ):
            I_msvc   = nucleotide.component.windows.msvc.translator.Translator()
            self.m_list += I_msvc.get()

        if( 'CYGWIN_NT' in platform.system() ):
            I_cygwin    = nucleotide.component.windows.mingw.translator.Translator()
            self.m_list += I_cygwin.get()

    def get(self):
        return self.m_list

    def check(self):
        pass

    @staticmethod
    def extend(P_options):

        if( False == Translator._detect() ):
            return

        nucleotide.component.windows._common.translator.Translator.extend(P_options)
        nucleotide.component.windows.mingw.translator.Translator.extend(P_options)

        if( 'Windows' == platform.system() ):
            nucleotide.component.windows.msvc.translator.Translator.extend(P_options)

        if( 'CYGWIN_NT' in platform.system() ):
            nucleotide.component.windows.cygwingcc.translator.Translator.extend(P_options)

    @staticmethod
    def _detect():
        if( 'Windows' == platform.system() ):
            #print( "Platform: " + platform.system() )
            return True

        if( 'CYGWIN_NT' in platform.system() ):
            #print( "Platform: " + platform.system() )
            return True

        print( "Unknown Platform: " + platform.system() )
        return False



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

#import nucleotide.component.linux.gcc.atom.module.boost
import nucleotide.component.linux.gcc.atom.module.opencv
#import nucleotide.component.linux.gcc.atom.module.zlib
#import nucleotide.component.linux.gcc.atom.module.tbb
#import nucleotide.component.linux.gcc.atom.module.protobuf


def _linux_gcc_atom_package_CPPDEFINES( P_list ):

    #if( True == ( 'boost' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.boost._linux_gcc_atom_module_boost_CPPDEFINES( P_list['boost'] )

    if( True == ( 'opencv' in  P_list ) ):
        return nucleotide.component.linux.gcc.atom.module.opencv._component_linux_gcc_atom_module_opencv_CPPDEFINES( P_list['opencv'] )

    #if( True == ( 'zlib' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.zlib._linux_gcc_atom_module_zlib_CPPDEFINES( P_list['zlib'] )
    #
    #if( True == ( 'tbb' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.tbb._linux_gcc_atom_module_tbb_CPPDEFINES( P_list['tbb'] )
    #
    #if( True == ( 'protobuf' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.protobuf._linux_gcc_atom_module_protobuf_CPPDEFINES( P_list['protobuf'] )

    return []

def _linux_gcc_atom_package_CPPPATH( P_list ):

    #if( True == ( 'boost' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.boost._linux_gcc_atom_module_boost_CPPPATH( P_list['boost'] )

    if( True == ( 'opencv' in  P_list ) ):
        return nucleotide.component.linux.gcc.atom.module.opencv._component_linux_gcc_atom_module_opencv_CPPPATH( P_list['opencv'] )

    #if( True == ( 'zlib' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.zlib._linux_gcc_atom_module_zlib_CPPPATH( P_list['zlib'] )
    #
    #if( True == ( 'tbb' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.tbb._linux_gcc_atom_module_tbb_CPPPATH( P_list['tbb'] )
    #
    #if( True == ( 'protobuf' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.protobuf._linux_gcc_atom_module_protobuf_CPPPATH( P_list['protobuf'] )

    return []

def _linux_gcc_atom_package_LINKFLAGS( P_list ):

    #if( True == ( 'boost' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.boost._linux_gcc_atom_module_boost_LINKFLAGS( P_list['boost'] )

    if( True == ( 'opencv' in  P_list ) ):
        return nucleotide.component.linux.gcc.atom.module.opencv._component_linux_gcc_atom_module_opencv_LINKFLAGS( P_list['opencv'] )

    #if( True == ( 'zlib' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.zlib._linux_gcc_atom_module_zlib_LINKFLAGS( P_list['zlib'] )
    #
    #if( True == ( 'tbb' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.tbb._linux_gcc_atom_module_tbb_LINKFLAGS( P_list['tbb'] )
    #
    #if( True == ( 'protobuf' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.protobuf._linux_gcc_atom_module_protobuf_LINKFLAGS( P_list['protobuf'] )

    return []

def _linux_gcc_atom_package_LIBPATH( P_list ):

    #if( True == ( 'boost' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.boost._linux_gcc_atom_module_boost_LIBPATH( P_list['boost'] )

    if( True == ( 'opencv' in  P_list ) ):
        return nucleotide.component.linux.gcc.atom.module.opencv._component_linux_gcc_atom_module_opencv_LIBPATH( P_list['opencv'] )

    #if( True == ( 'zlib' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.zlib._linux_gcc_atom_module_zlib_LIBPATH( P_list['zlib'] )
    #
    #if( True == ( 'tbb' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.tbb._linux_gcc_atom_module_tbb_LIBPATH( P_list['tbb'] )
    #
    #if( True == ( 'protobuf' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.protobuf._linux_gcc_atom_module_protobuf_LIBPATH( P_list['protobuf'] )

    return []

def _linux_gcc_atom_package_LIBS( P_list ):

    #if( True == ( 'boost' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.boost._linux_gcc_atom_module_boost_LIBS( P_list['boost'] )

    if( True == ( 'opencv' in  P_list ) ):
        return nucleotide.component.linux.gcc.atom.module.opencv._component_linux_gcc_atom_module_opencv_LIBS( P_list['opencv'] )

    #if( True == ( 'zlib' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.zlib._linux_gcc_atom_module_zlib_LIBS( P_list['zlib'] )
    #
    #if( True == ( 'tbb' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.tbb._linux_gcc_atom_module_tbb_LIBS( P_list['tbb'] )
    #
    #if( True == ( 'protobuf' in  P_list ) ):
    #    return nucleotide.component.linux.gcc.atom.module.protobuf._linux_gcc_atom_module_protobuf_LIBS( P_list['protobuf'] )

    return []


atom__common_package = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPDEFINES' : _linux_gcc_atom_package_CPPDEFINES,
        'CPPPATH'    : _linux_gcc_atom_package_CPPPATH,
        'LINKFLAGS'  : _linux_gcc_atom_package_LINKFLAGS,
        'LIBPATH'    : _linux_gcc_atom_package_LIBPATH,
        'LIBS'       : _linux_gcc_atom_package_LIBS,
    },

    'name' :'package',
    'class':  [ 'package', 'linux:package' ]
}

class Package:

    def __init( self ) :
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'A:linux:package',               atom__common_package)
        atom__common_package['platform']['host']='X'
        nucleotide.component.function.extend( P_option, 'x:linux:package',               atom__common_package)
        atom__common_package['platform']['guest']='X'
        nucleotide.component.function.extend( P_option, 'y:linux:package',               atom__common_package)

    @staticmethod
    def check():
        pass


#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _common_package_CPPDEFINES( P_list ):
    Ir_list = []
    return Ir_list

def _common_package_CPPPATH( P_list ):
    Ir_list = []
    return Ir_list

def _common_package_LINKFLAGS( P_list ):
    Ir_list = []
    return Ir_list

def _common_package_LIBPATH( P_list ):
    Ir_list = []
    return Ir_list

def _common_package_LIBS( P_list ):
    Ir_list = []
    return Ir_list


atom__common_package = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor' : 'X',
        'name'   : 'X',
        'version': 'X'
    },
    'config' : {
        'CPPDEFINES' : _common_package_CPPDEFINES,
        'CPPPATH'    : _common_package_CPPPATH,
        'LINKFLAGS'  : _common_package_LINKFLAGS,
        'LIBPATH'    : _common_package_LIBPATH,
        'LIBS'       : _common_package_LIBS,
    },

    'name' :'package',
    'class':  [ 'package' ]
}

class Package:

    def __init( self ) :
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'package',    atom__common_package )

    @staticmethod
    def check():
        pass


#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

import nucleotide.component.windows.msvc.atom.module.boost
import nucleotide.component.windows.msvc.atom.module.opencv
import nucleotide.component.windows.msvc.atom.module.zlib
import nucleotide.component.windows.msvc.atom.module.tbb
import nucleotide.component.windows.msvc.atom.module.protobuf
import nucleotide.component.windows.msvc.atom.module.python
import nucleotide.component.windows.msvc.atom.module.bzip2


Is_list={
    'boost': {
        'CPPDEFINES':nucleotide.component.windows.msvc.atom.module.boost._windows_msvc_atom_module_boost_CPPDEFINES,
        'CPPPATH'   :nucleotide.component.windows.msvc.atom.module.boost._windows_msvc_atom_module_boost_CPPPATH,
        'LINKFLAGS' :nucleotide.component.windows.msvc.atom.module.boost._windows_msvc_atom_module_boost_LINKFLAGS,
        'LIBPATH'   :nucleotide.component.windows.msvc.atom.module.boost._windows_msvc_atom_module_boost_LIBPATH,
        'LIBS'      :nucleotide.component.windows.msvc.atom.module.boost._windows_msvc_atom_module_boost_LIBS,
    },
    'opencv': {
        'CPPDEFINES':nucleotide.component.windows.msvc.atom.module.opencv._windows_msvc_atom_module_opencv_CPPDEFINES,
        'CPPPATH'   :nucleotide.component.windows.msvc.atom.module.opencv._windows_msvc_atom_module_opencv_CPPPATH,
        'LINKFLAGS' :nucleotide.component.windows.msvc.atom.module.opencv._windows_msvc_atom_module_opencv_LINKFLAGS,
        'LIBPATH'   :nucleotide.component.windows.msvc.atom.module.opencv._windows_msvc_atom_module_opencv_LIBPATH,
        'LIBS'      :nucleotide.component.windows.msvc.atom.module.opencv._windows_msvc_atom_module_opencv_LIBS,
    },
    'zlib': {
        'CPPDEFINES':nucleotide.component.windows.msvc.atom.module.zlib._windows_msvc_atom_module_zlib_CPPDEFINES,
        'CPPPATH'   :nucleotide.component.windows.msvc.atom.module.zlib._windows_msvc_atom_module_zlib_CPPPATH,
        'LINKFLAGS' :nucleotide.component.windows.msvc.atom.module.zlib._windows_msvc_atom_module_zlib_LINKFLAGS,
        'LIBPATH'   :nucleotide.component.windows.msvc.atom.module.zlib._windows_msvc_atom_module_zlib_LIBPATH,
        'LIBS'      :nucleotide.component.windows.msvc.atom.module.zlib._windows_msvc_atom_module_zlib_LIBS,
    },

    'tbb': {
        'CPPDEFINES':nucleotide.component.windows.msvc.atom.module.tbb._windows_msvc_atom_module_tbb_CPPDEFINES,
        'CPPPATH'   :nucleotide.component.windows.msvc.atom.module.tbb._windows_msvc_atom_module_tbb_CPPPATH,
        'LINKFLAGS' :nucleotide.component.windows.msvc.atom.module.tbb._windows_msvc_atom_module_tbb_LINKFLAGS,
        'LIBPATH'   :nucleotide.component.windows.msvc.atom.module.tbb._windows_msvc_atom_module_tbb_LIBPATH,
        'LIBS'      :nucleotide.component.windows.msvc.atom.module.tbb._windows_msvc_atom_module_tbb_LIBS,
    },

    'protobuf': {
        'CPPDEFINES':nucleotide.component.windows.msvc.atom.module.protobuf._windows_msvc_atom_module_protobuf_CPPDEFINES,
        'CPPPATH'   :nucleotide.component.windows.msvc.atom.module.protobuf._windows_msvc_atom_module_protobuf_CPPPATH,
        'LINKFLAGS' :nucleotide.component.windows.msvc.atom.module.protobuf._windows_msvc_atom_module_protobuf_LINKFLAGS,
        'LIBPATH'   :nucleotide.component.windows.msvc.atom.module.protobuf._windows_msvc_atom_module_protobuf_LIBPATH,
        'LIBS'      :nucleotide.component.windows.msvc.atom.module.protobuf._windows_msvc_atom_module_protobuf_LIBS,
    },

    'python': {
        'CPPDEFINES':nucleotide.component.windows.msvc.atom.module.python._windows_msvc_atom_module_python_CPPDEFINES,
        'CPPPATH'   :nucleotide.component.windows.msvc.atom.module.python._windows_msvc_atom_module_python_CPPPATH,
        'LINKFLAGS' :nucleotide.component.windows.msvc.atom.module.python._windows_msvc_atom_module_python_LINKFLAGS,
        'LIBPATH'   :nucleotide.component.windows.msvc.atom.module.python._windows_msvc_atom_module_python_LIBPATH,
        'LIBS'      :nucleotide.component.windows.msvc.atom.module.python._windows_msvc_atom_module_python_LIBS,
    },

    'bzip2': {
        'CPPDEFINES':nucleotide.component.windows.msvc.atom.module.bzip2._windows_msvc_atom_module_bzip2_CPPDEFINES,
        'CPPPATH'   :nucleotide.component.windows.msvc.atom.module.bzip2._windows_msvc_atom_module_bzip2_CPPPATH,
        'LINKFLAGS' :nucleotide.component.windows.msvc.atom.module.bzip2._windows_msvc_atom_module_bzip2_LINKFLAGS,
        'LIBPATH'   :nucleotide.component.windows.msvc.atom.module.bzip2._windows_msvc_atom_module_bzip2_LIBPATH,
        'LIBS'      :nucleotide.component.windows.msvc.atom.module.bzip2._windows_msvc_atom_module_bzip2_LIBS,
    }
}

def _windows_msvc_atom_package_CPPDEFINES( P_list ):

    #print( P_list )
    for key in P_list:
        if( key in Is_list ):
            return Is_list[key]['CPPDEFINES'](  P_list[key] )
        else:
            print( 'Pakage: \'' + key + '\' Not found.' )

    return []


def _windows_msvc_atom_package_CPPPATH( P_list ):

    #print( P_list )
    for key in P_list:
        if( key in Is_list ):
            return Is_list[key]['CPPPATH'](  P_list[key] )
        else:
            print( 'Pakage: \'' + key + '\' Not found.' )

    return []

def _windows_msvc_atom_package_LINKFLAGS( P_list ):

    #print( P_list )
    for key in P_list:
        if( key in Is_list ):
            return Is_list[key]['LINKFLAGS'](  P_list[key] )
        else:
            print( 'Pakage: \'' + key + '\' Not found.' )

    return []

def _windows_msvc_atom_package_LIBPATH( P_list ):

    #print( P_list )
    for key in P_list:
        if( key in Is_list ):
            return Is_list[key]['LIBPATH'](  P_list[key] )
        else:
            print( 'Pakage: \'' + key + '\' Not found.' )

    return []

def _windows_msvc_atom_package_LIBS( P_list ):

    #print( P_list )
    for key in P_list:
        if( key in Is_list ):
            return Is_list[key]['LIBS'](  P_list[key] )
        else:
            print( 'Pakage: \'' + key + '\' Not found.' )

    return []


atom__common_package = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor' : 'Microsoft',
        'name'   : 'msvc',
        'version': 'X'
    },
    'config' : {
        'CPPDEFINES' : _windows_msvc_atom_package_CPPDEFINES,
        'CPPPATH'    : _windows_msvc_atom_package_CPPPATH,
        'LINKFLAGS'  : _windows_msvc_atom_package_LINKFLAGS,
        'LIBPATH'    : _windows_msvc_atom_package_LIBPATH,
        'LIBS'       : _windows_msvc_atom_package_LIBS,
    },

    'name' :'package',
    'class':  [ 'package', 'windows:package' ]
}

class Package:

    def __init( self ) :
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'package',    atom__common_package )

    @staticmethod
    def check():
        pass


#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _component_linux_gcc_atom_module_opencv_CPPDEFINES( P_data ):
    Ir_list = ["OPENCV"]

    return Ir_list

def _component_linux_gcc_atom_module_opencv_CPPPATH( P_data ):
    Ir_list = ['/usr/local/include/opencv', '/usr/local/include/opencv2']
    return Ir_list

def _component_linux_gcc_atom_module_opencv_LINKFLAGS( P_data ):
    Ir_list = []

    return Ir_list

def _component_linux_gcc_atom_module_opencv_LIBPATH( P_data ):
    Ir_list = [ "/usr/local/lib/", "/usr/lib" ]

    return Ir_list

def _component_linux_gcc_atom_module_opencv_LIBS( P_data ):
    Ir_list = [ "libopencv_core.dll", "opencv_highgui.dll", "opencv_imgproc.dll", "opencv_videoio.dll", "opencv_imgproc.dll" ]

    return Ir_list

atom_linux_configuration = {
    'platform' : {
        'host'  : 'Linux',
        'guest' : 'Linux'
    },
    'cc' : {
        'vendor' : 'FSF',
        'name'   : 'gcc',
        'version': 'X'
    },
    'config' : {
        'CPPDEFINES' : _component_linux_gcc_atom_module_opencv_CPPDEFINES,
        'CPPPATH'    : _component_linux_gcc_atom_module_opencv_CPPPATH,
        'LINKFLAGS'  : _component_linux_gcc_atom_module_opencv_LINKFLAGS,
        'LIBPATH'    : _component_linux_gcc_atom_module_opencv_LIBPATH,
        'LIBS'       : _component_linux_gcc_atom_module_opencv_LIBS,
    },
    'name' :'package',
    'class':  [ 'package::opencv', 'linux:package:opencv' ]
}

class Configuration:
    def __init__(self):
        pass

    @staticmethod
    def extend( P_option ):
         nucleotide.component.function.extend( P_option, 'A:linux:compiler:configuration',               atom_linux_configuration )
         atom_linux_configuration['platform']['host']='X'
         nucleotide.component.function.extend( P_option, 'x:linux:compiler:configuration',               atom_linux_configuration )
         atom_linux_configuration['platform']['guest']='X'
         nucleotide.component.function.extend( P_option, 'y:linux:compiler:configuration',               atom_linux_configuration )

    @staticmethod
    def check():
        pass



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import sys
import platform

import nucleotide
import nucleotide.component
import nucleotide.component.translator

##  Set of atoms grouped around represents
#   An atom can be connected with several represents at the same time
# Answer on Questions: What I can do? or What is possibilities?
class Options:
    m_this = None
    m_represent = None

    def __init__( self, P_enumerate = True ):
        self.m_this = {}
        self.m_represent = {}

        #self.m_this['blank'] = nucleotide.atom.Atom( )

        if( True == P_enumerate ):
            self._enumerate( P_enumerate )

    @staticmethod
    def check():
        #nucleotide.component.translator.Translator.check()
        pass

    def _enumerate( self, P_enumerate = True ):
        if( False == P_enumerate ):
            return

        nucleotide.component.translator.Translator.extend( self )

    ##Extend current options with new atom
    def extend( self, P_name, P_atom ):
        #print( 'Options::extend ( P_name = ' + P_name + ', '+ ' )' )
        #print( '    Options::extend ( P_atom.get_translator().string() = ' + P_atom.get_translator().string() +' )' )
        #print( '    Options::extend ( P_atom.get_klass().others() = ' + str( P_atom.get_klass().others() ) +' )' )

        full_name = self.make_name( P_atom, P_name );
        self.m_this[ full_name ] = P_atom

        for represent in P_atom.get_klass().others():
            if( False == ( represent in self.m_represent ) ):
                self.m_represent[ represent ]  = []
            self.m_represent[ represent ] += [ full_name ]
            #print( '    Options::extend( represent: ' + represent  + '   with: '+ full_name+' )' )

    ##Return specific atom for given translator and universal name
    def get( self, P_translator, P_universal ):
        #print( 'Options::get(' + ' translator =' + P_translator.string()  + ', universal = '+ P_universal + ' )' )

        if( False == ( P_universal in self.m_represent ) ) :
            print( '    Options::get:: 0 - i_classeq = [] for \'' + P_universal + '\'' )
            return nucleotide.atom.Atom( )

        i_classeq = self.m_represent[ P_universal ]
        I_best = { 'level': -1, 'name': 'X' }
        #print( '    Options::get:: 1 - i_classeq = ' + str( i_classeq ) )

        for element in i_classeq:
            if( False == ( element in self.m_this ) ) :
                #print( '            Options::get:: 2 - no key = ' + str( element ) )
                continue
            I_level =  P_translator.similarity( self.m_this[ element ].get_translator() )
            if( I_best['level'] < I_level ):
                I_best['level'] = I_level
                I_best['name'] = element
            #print( '    Options::get:: 3 - ' + ' element:   ' + element + '; ' + ' level: ' + str( I_level ) )

        if( -1 == I_best['level'] ):
            print( '    Options::get:: 4 for: ' + '||' + str( I_best['level'] ) + '|| - ' + P_universal + " - " + I_best['name'] )
            return nucleotide.atom.Atom( )

        Options.message( '    Options::get:: 5 for: ' + '||' + str( I_best['level'] ) + '|| - ' + P_universal + " - " + I_best['name'] )
        return self.m_this[ I_best['name'] ]

    def get_this(self):
        return self.m_this

    def get_represents(self):
        return self.m_represent

    @staticmethod
    def message(P_message):
        print( P_message )

    def join_out2(self, a, b ):
        return a + '::'  + b

    def join_in2(a,b):
        return a + separator_in  + b

    def join_in4(a,b,c,d):
        return a + separator_in  + b+ separator_in  + c + separator_in  + d

    def join_in6(a,b,c,d,e):
        return a + separator_in  + b+ separator_in  + c + separator_in  + d + separator_in  + e

    def make_name( self,P_atom, P_name ):
        return self.join_out2( P_atom.get_translator().string(), P_name )




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import subprocess

import nucleotide
import nucleotide.component
import nucleotide.component.function


def _python_python27_mingw_CPPPATH( P_list ):
    I_result = os.getenv('PYTHON35_MINGW_INCLUDE')
    if( None == I_result ):
        return nucleotide.component.windows._common.python._python27_default_CPPPATH( P_list )
    return [ I_result ]

def _python_python27_mingw_LIBPATH( P_list ):
    I_result = os.getenv('PYTHON35_MINGW_LIBPATH')
    if( None == I_result ):
        return nucleotide.component.windows._common.python._python27_default_LIBPATH( P_list )
    return [ I_result ]

atom_python27_mingw = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor'  : 'mingw',
        'name'    : 'gcc',
        'version' : 'X'
    },
    'config' : {
            'CPPPATH': _python_python27_mingw_CPPPATH,
            'LIBPATH': _python_python27_mingw_LIBPATH,
            'LIBS'   : [ 'python27.a' ]
    },
    'name' : 'python27:mingw',
    'class':  [ 'python', 'python:default', 'python27:default', 'python27:mingw' ]
}


def _python_python35_mingw_CPPPATH( P_list ):
    I_result = os.getenv( 'PYTHON35_MINGW_INCLUDE' )
    if( None == I_result ):
        return nucleotide.component.windows._common.python._python35_default_CPPPATH( P_list )
    return [ I_result ]

def _python_python35_mingw_LIBPATH( P_list ):
    I_result = os.getenv( 'PYTHON35_MINGW_LIBPATH' )
    if( None == I_result ):
        return nucleotide.component.windows._common.python._python35_default_LIBPATH( P_list )
    return [ I_result ]

atom_python35_mingw = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor'  : 'mingw',
        'name'    : 'gcc',
        'version' : 'X'
    },
    'config' : {
            'CPPPATH': _python_python35_mingw_CPPPATH,
            'LIBPATH': _python_python35_mingw_LIBPATH,
            'LIBS'   : [ 'python35.a' ]
    },
    'name' : 'python35:mingw',
    'class':  [ 'python', 'python:default', 'python35:default', 'python35:mingw' ]
}

class Python:
    def __init__( ):
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'python27:mingw ',  atom_python27_mingw   )
        nucleotide.component.function.extend( P_option, 'python35:mingw',   atom_python35_mingw   )

    def check():

        nucleotide.component.function.check__env( 'PYTHONPATH' )

        nucleotide.component.function.check__env( 'PYTHON27_MINGW_INCLUDE' )
        nucleotide.component.function.check__env( 'PYTHON27_MINGW_LIBPATH' )
        nucleotide.component.function.check__env( 'PYTHON27_MINGW_VERSION' )

        nucleotide.component.function.check__env( 'PYTHON35_MINGW_INCLUDE' )
        nucleotide.component.function.check__env( 'PYTHON35_MINGW_LIBPATH' )
        nucleotide.component.function.check__env( 'PYTHON35_MINGW_VERSION' )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import platform
import copy

_winreg_HKEY_LOCAL_MACHINE = 0
_winreg_OpenKey  = None
_winreg_CloseKey = None

try:
    import _winreg
    _winreg_HKEY_LOCAL_MACHINE = _winreg.HKEY_LOCAL_MACHINE
    _winreg_OpenKey  = _winreg.OpenKey
    _winreg_CloseKey = _winreg.CloseKey
except ImportError:
    try:
        import winreg
        _winreg_HKEY_LOCAL_MACHINE = winreg.HKEY_LOCAL_MACHINE
        _winreg_OpenKey  = winreg.OpenKey
        _winreg_CloseKey = winreg.CloseKey
    except ImportError:
        _winreg_OpenKey  = None
        _winreg_CloseKey = None

import nucleotide
import nucleotide.translator
import nucleotide.component
import nucleotide.component.windows
import nucleotide.component.windows.msvc
import nucleotide.component.windows.msvc.atom
import nucleotide.component.windows.msvc.atom._misc
import nucleotide.component.windows.msvc.atom.python
import nucleotide.component.windows.msvc.atom.warning
import nucleotide.component.windows.msvc.atom.rtti
import nucleotide.component.windows.msvc.atom.rtl
import nucleotide.component.windows.msvc.atom.pdb
import nucleotide.component.windows.msvc.atom.version
import nucleotide.component.windows.msvc.atom.blank
import nucleotide.component.windows.msvc.atom.dialect
import nucleotide.component.windows.msvc.atom.pp2f
import nucleotide.component.windows.msvc.atom.executable
import nucleotide.component.windows.msvc.atom.exception
import nucleotide.component.windows.msvc.atom.encode
import nucleotide.component.windows.msvc.atom.macro
import nucleotide.component.windows.msvc.atom.package
import nucleotide.component.windows.msvc.atom.rebuild_lazy
import nucleotide.component.windows.msvc.atom.alignment
import nucleotide.component.windows.msvc.atom.optimization


##Detect MSVC on Window
class Translator:
    m_list = []
    def __init__(self):
        self.m_list = []

        I_data = nucleotide.Translator.blank()
        I_data['platform']['host'] = 'Windows'
        I_data['platform']['guest'] = 'Windows'
        I_data['cc']['vendor'] = 'Microsoft'
        I_data['cc']['name'] = 'msvc'

        if( True == Translator._exists( _winreg_HKEY_LOCAL_MACHINE, 'SOFTWARE\\Microsoft\\VisualStudio\\14.0' ) ):
            I_data['cc']['version'] = '14'
            I_c = copy.deepcopy( I_data )
            self.m_list.append( nucleotide.Translator( I_c['platform'], I_c['cc'] ) )

        if( True == Translator._exists( _winreg_HKEY_LOCAL_MACHINE, 'SOFTWARE\\Microsoft\\VisualStudio\\12.0' ) ):
            I_data['cc']['version'] = '12'
            I_c = copy.deepcopy( I_data )
            self.m_list.append( nucleotide.Translator( I_c['platform'], I_c['cc'] ) )

        if( True == Translator._exists( _winreg_HKEY_LOCAL_MACHINE, 'SOFTWARE\\Microsoft\\VisualStudio\\11.0' ) ):
            I_data['cc']['version'] = '11'
            I_c = copy.deepcopy( I_data )
            self.m_list.append( nucleotide.Translator( I_c['platform'], I_c['cc'] ) )

        if( True == Translator._exists( _winreg_HKEY_LOCAL_MACHINE, 'SOFTWARE\\Microsoft\\VisualStudio\\10.0' ) ):
            I_data['cc']['version'] = '10'
            I_c = copy.deepcopy( I_data )
            self.m_list.append( nucleotide.Translator( I_c['platform'], I_c['cc'] ) )

        if( True == Translator._exists( _winreg_HKEY_LOCAL_MACHINE, 'SOFTWARE\\Microsoft\\VisualStudio\\9.0' ) ):
            I_data['cc']['version'] = '9'
            I_c = copy.deepcopy( I_data )
            self.m_list.append( nucleotide.Translator( I_c['platform'], I_c['cc'] ) )

    def get(self):
        return self.m_list

    @staticmethod
    def check():
        return True

    @staticmethod
    def extend(P_options):
        nucleotide.component.windows.msvc.atom._misc.init(P_options)
        nucleotide.component.windows.msvc.atom.python.Python.extend(P_options)
        nucleotide.component.windows.msvc.atom.warning.Warning.extend(P_options)

        nucleotide.component.windows.msvc.atom.rtti.RTTI.extend(P_options)
        nucleotide.component.windows.msvc.atom.rtl.RTL.extend(P_options)
        nucleotide.component.windows.msvc.atom.pdb.PDB.extend(P_options)

        nucleotide.component.windows.msvc.atom.version.Version.extend(P_options)
        nucleotide.component.windows.msvc.atom.blank.Blank.extend(P_options)
        nucleotide.component.windows.msvc.atom.dialect.Dialect.extend(P_options)
        nucleotide.component.windows.msvc.atom.pp2f.PP2F.extend(P_options)
        nucleotide.component.windows.msvc.atom.executable.Executable.extend(P_options)
        nucleotide.component.windows.msvc.atom.exception.Exception.extend(P_options)
        nucleotide.component.windows.msvc.atom.encode.Encode.extend(P_options)
        nucleotide.component.windows.msvc.atom.macro.Macro.extend(P_options)
        nucleotide.component.windows.msvc.atom.package.Package.extend(P_options)
        nucleotide.component.windows.msvc.atom.rebuild_lazy.RebuildLazy.extend(P_options)
        nucleotide.component.windows.msvc.atom.alignment.Alignment.extend(P_options)
        nucleotide.component.windows.msvc.atom.optimization.Optimization.extend(P_options)

    @staticmethod
    def _exists( key, sub_key ):
        if None == _winreg_OpenKey:
           result = False
           return result

        try:
            k = _winreg_OpenKey( key, sub_key )
            _winreg_CloseKey( k )
            result = True
        except WindowsError:
            result = False
        return result



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import platform

import nucleotide
import nucleotide.component._common
import nucleotide.component._common.warning
import nucleotide.component._common.warning.table

def windows_make( P_native, P_status ): # https://msdn.microsoft.com/en-us/library/thxezb7y.aspx
    Ir_switch = ''
    if( 'enable'   == P_status ): Ir_switch = '/w1' + P_native
    if( 'disable'  == P_status ): Ir_switch = '/wd' + P_native
    if( 'error'    == P_status ): Ir_switch = '/we' + P_native
    if( 'once'     == P_status ): Ir_switch = '/wo' + P_native
    return Ir_switch

def linux_make( P_native, P_status ):
    Ir_switch = ''
    if( 'enable'   == P_status ): Ir_switch = '-W'       + P_native
    if( 'disable'  == P_status ): Ir_switch = '-Wno-'    + P_native
    if( 'error'    == P_status ): Ir_switch = '-Werror=' + P_native
    if( 'once'     == P_status ): Ir_switch = ''
    return Ir_switch

def convertor(P_translator):
    if( 'Linux'   == platform.system() ):
        return linux_make

    if( 'Windows' == platform.system() ):
        return windows_make

    if( 'CYGWIN_NT' in platform.system() ):
        return linux_make

    print( "Unknown Platform : " + platform.system() )
    return None

def list( P_list, P_translator ):
    Ir_list = []
    I_convertor = convertor( P_translator )

    if( None == I_convertor ):
        print( "Non convertor" )
        return Ir_list;

    for item in P_list:
        if( False == ( item in nucleotide.component._common.warning.table.TABLE ) ):
            continue
        I_best = -1
        for elem in nucleotide.component._common.warning.table.TABLE[item]:
            I_similar = P_translator.similarity( nucleotide.translator.Translator.extract( elem ) )
            if( I_best < I_similar ):
                I_best = I_similar
                I_text = nucleotide.component._common.warning.table.TABLE[item][elem]
        if( -1 != I_best ):
            Ir_list += [ I_convertor( I_text, P_list[item] ) ]
        else:
            pass
        print( "        ||" + str(I_best) + '|| - ' + str( Ir_list ) )

    return Ir_list



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import os
import subprocess

import nucleotide
import nucleotide.component
import nucleotide.component.function

def _windows_PDB_CPPFLAGS( P_data ):
    Ir_list = []

    if( True == ( 'configuration' in P_data ) ):
        if( 'debug' == P_data[ 'configuration' ].lower() ):
           Ir_list.append( '/ZI' )
        if( 'release' == P_data[ 'configuration' ].lower() ):
           Ir_list.append( '/Zi' )

    if( True == ( 'format' in P_data ) ):
        if( 'none' == P_data[ 'format' ] ):
           pass
        if( 'C7' == P_data[ 'format' ] ):
           Ir_list.append( '/Z7' )
        if( 'classic' == P_data[ 'format' ] ):
           Ir_list.append( '/Zi' )
        if( 'extended' == P_data[ 'format' ] ):
           Ir_list.append( '/ZI' )

    if( True == ( 'synchronous' in P_data ) ):
        if( 'true' == P_data[ 'synchronous' ] ):
           Ir_list.append( '/FS' )

    if( True == ( 'file-name-compile' in P_data ) ):
        Ir_list.append( '/Fd"' + P_data['file-name-compile'] + '"' )
    else:
        if( True == ( 'file-name' in P_data ) ):
            Ir_list.append( '/Fd"' + P_data['file-name'] + '"' )

    return Ir_list

def _windows_PDB_LINKFLAGS( P_data ):
    Ir_list = []
    if( True == ( 'configuration' in P_data ) ):
        if( 'debug' == P_data['configuration'].lower() ):
           Ir_list.append( '/DEBUG' )
        if( 'release' == P_data['configuration'].lower() ):
           Ir_list.append( '/DEBUG' )

    if( True == ( 'file-name-executable' in P_data )  ):
        Ir_list.append( '/PDB:"' + P_data['file-name-executable'] + '"' )
    else:
        if( True == ( 'file-name' in P_data ) ):
            Ir_list.append( '/PDB:"' + P_data['file-name'] + '"' )

    return Ir_list

atom_windows_PDB = {
    'platform' : {
        'host'  : 'Windows',
        'guest' : 'Windows'
    },
    'cc' : {
        'vendor': 'Microsoft',
        'name': 'msvc',
        'version': 'X'
    },
    'config' : {
        'CPPFLAGS'  : _windows_PDB_CPPFLAGS,
        'LINKFLAGS' : _windows_PDB_LINKFLAGS,
    },
    'name' : 'PDB',
    'class': [  'PDB', 'windows:PDB' ]
}


class PDB:
    def __init__(self):
        pass

    @staticmethod
    def extend(P_option):
        nucleotide.component.function.extend( P_option, 'windows:PDB',  atom_windows_PDB )

    @staticmethod
    def check(self):
        pass




#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License. 


import os
import subprocess

import nucleotide
import nucleotide.component
import nucleotide.component.function



def _python_default_CPPPATH( P_list ):
    return [ os.getenv('PYTHON_INCLUDE') ]

def _python_default_LIBPATH( P_list ):
    return [ os.getenv('PYTHON_LIBPATH') ]

def _python_default_LIBS( P_list ):
    #TODO 'python' + P_list['version']
    return [ 'pythonXYZ' ]

atom_python_default = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor' : 'X',
        'name'   : 'X',
        'version': 'X'
    },
    'config' : {
            'CPPPATH'  : _python_default_CPPPATH,
            'LIBPATH'  : _python_default_LIBPATH,
            'LIBS'     : _python_default_LIBS
    },
    'name' : 'python:default',
    'class': [ 'python', 'python:default' ]
}


def _python27_default_CPPPATH( P_list ):
    I_result = os.getenv('PYTHON27_INCLUDE')
    if( None == I_result ):
        return _python_default_CPPPATH( P_list )
    return [ I_result ]

def _python27_default_LIBPATH( P_list ):
    I_result = os.getenv('PYTHON27_LIBPATH')
    if( None == I_result ):
        return _python_default_LIBPATH( P_list )
    return [ I_result ]

atom_python27_default = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor'  : 'X',
        'name'    : 'X',
        'version' : 'X'
    },
    'config' : {
            'CPPPATH'  : _python27_default_CPPPATH,
            'LIBPATH'  : _python27_default_LIBPATH,
            'LIBS'   : [ 'python27' ]
    },
    'name' : 'python27:default',
    'class':  [ 'python', 'python:default', 'python27:default' ]
}


def _python35_default_CPPPATH( P_list ):
    I_result = os.getenv('PYTHON35_INCLUDE')
    if( None == I_result ):
        return _python_default_CPPPATH( P_list )
    return [ I_result ]

def _python35_default_LIBPATH( P_list ):
    I_result = os.getenv('PYTHON35_LIBPATH')
    if( None == I_result ):
        return _python_default_LIBPATH( P_list )
    return [ I_result ]

atom_python35_default = {
    'platform' : {
        'host'  : 'X',
        'guest' : 'X'
    },
    'cc' : {
        'vendor': 'X',
        'name': 'X',
        'version': 'X'
    },
    'config' : {
            'CPPPATH'  : _python35_default_CPPPATH,
            'LIBPATH'  : _python35_default_LIBPATH,
            'LIBS'   : [ 'python35' ]
    },
    'name' : 'python35:default',
    'class': [  'python', 'python:default', 'python35:default' ]
}

class Python:
    def __init__( self ):
        pass

    @staticmethod
    def extend( P_option ) :
        nucleotide.component.function.extend( P_option, 'python:default',    atom_python_default   )
        nucleotide.component.function.extend( P_option, 'python27:default',  atom_python27_default )
        nucleotide.component.function.extend( P_option, 'python35:default',  atom_python35_default )

    @staticmethod
    def check():
        nucleotide.component.function.check__env( 'PYTHONPATH' )
        nucleotide.component.function.check__env( 'PYTHON_INCLUDE' )
        nucleotide.component.function.check__env( 'PYTHON_LIBPATH' )
        nucleotide.component.function.check__env( 'PYTHON_VERSION' )

        nucleotide.component.function.check__env( 'PYTHON27_INCLUDE' )
        nucleotide.component.function.check__env( 'PYTHON27_LIBPATH' )
        nucleotide.component.function.check__env( 'PYTHON27_VERSION' )

        nucleotide.component.function.check__env( 'PYTHON35_INCLUDE' )
        nucleotide.component.function.check__env( 'PYTHON35_LIBPATH' )
        nucleotide.component.function.check__env( 'PYTHON35_VERSION' )



#!/usr/bin/env python2

#   Copyright 2015 Dejan D. M. Milosavljevic
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


TABLE = {
    'misleading-indentation'                : {
            'X-X-FSF-gcc-X': 'misleading-indentation'
        },
    'unused-variable'                       : {
            'X-X-FSF-gcc-X'                   : 'unused-variable',
            'Windows-Windows-Microsoft-msvc-X': '4101'
        },
    'unused-variable-but-set'               : {
            'X-X-FSF-gcc-X'                   : 'unused-but-set-variable',
            'Windows-Windows-Microsoft-msvc-X': '4189'
        },
    'unused-local-typedefs'                 : {
            'X-X-FSF-gcc-X'                   :  'unused-local-typedefs',
            'Windows-Windows-Microsoft-msvc-X': '4901'
        },
    'unused-formal-parameter'               : {
            'Windows-Windows-Microsoft-msvc-X': '4100'
        },
    'unused-code':{
            'Windows-Windows-Microsoft-msvc-X': '4702'
        },

     'member-uninitialized'           : {
            #'X-X-FSF-gcc-X': 'TODO'
            #'Windows-Windows-Microsoft-msvc-X': 'TODO'
        },

    'member-order-initialization'           : {
            'X-X-FSF-gcc-X': 'reorder',
            'Windows-Windows-Microsoft-msvc-X': '5038'
        },

    'return-no-return-statement'            : {
            'X-X-FSF-gcc-X': 'return-type'
        },
    'return-address-of-local-variable': {
            'Windows-Windows-Microsoft-msvc-X': '4172',
            'X-X-FSF-gcc-X'                   : 'return-local-addr'
        },

    'uninitialized-variable'                : {
            'X-X-FSF-gcc-X': 'uninitialized'
        },
    'uninitialized-using-of-variable' : {
            'Windows-Windows-Microsoft-msvc-X': '4700',
            'X-X-FSF-gcc-X'                   : 'uninitialized'
        },

    'base-class-destructor-is-inaccessible' : {
            'Windows-Windows-Microsoft-msvc-X': '4624'
        },

    'conversion-comparation' : {
        'Windows-Windows-Microsoft-msvc-X': [ '4388', '4018' ],
        'linux-gcc'                       : 'sign-compare',
        'X-X-FSF-gcc-X'                   : 'sign-compare'
        },

    'conversion-data-loss' : {
        'Windows-Windows-Microsoft-msvc-X': '4242'
        },

    'conversion-force-to-bool' : {
        'Windows-Windows-Microsoft-msvc-X': '4800'
        },

    'enumerator-not-explicitly-handled' : {
        'Windows-Windows-Microsoft-msvc-X': '4601'
        },
    'enumerator-not-handled' : {
        'Windows-Windows-Microsoft-msvc-X': '4602'
        },

    'this-in-initializer-list' : {
        'Windows-Windows-Microsoft-msvc-X': '4355'
        },

    'macro-not-enought-parameter' : {
        'Windows-Windows-Microsoft-msvc-X': '4003'
        },
    'macro-redfinition' : {
        'Windows-Windows-Microsoft-msvc-X': '4005'
        },

    'decorated-name-length-exceeded' : {
        'Windows-Windows-Microsoft-msvc-X': '4503'
    },

    'assignment-within-conditional-expression': {
        'Windows-Windows-Microsoft-msvc-X': '4706',
        'X-X-FSF-gcc-X'                   : 'sign-compare'
    },

    'variable-shadow-variable' : {
    'linux-gcc': 'shadow'
    }

}
